/*     Autodesk rmarble.sl edited by Rand Williams CServe:76350,1765

	AnyMarble: Ka = ammount of ambience
			   Kd = ammount of diffusely reflected light
			   Ks = ammount of specularly reflected light
			   roughness = the relative area _high_lighted by Ks
			   veining = ammount of veins present, higher # -> more veins
			   marblecolor =  color of the marble surface
			   veincolor = color of the veins!
			   specularcolor = color of specular highlight
*/

surface
anymarble(
	float  Ka            = 0.1, 
		   Kd            = 0.6, 
		   Ks            = 0.4, 
		   roughness     = 0.1, 
		   veining       = 1.0;
	color  marblecolor   = 1,
		   veincolor     = 1,
		   specularcolor = 1
	   )
{
	varying point PPa;
	float cmia;
	varying point Va, Nfa ;
	color diffusecolora ;
	float pixelsizea, twicea, scalea, 
			weighta, turbulencea;
	float c,ca,cb,cc,cd;
	float red,green,blue,vred,vgreen,vblue;
	color mc,vc;

	Nfa = faceforward( normalize(N), I);
	Va = -normalize(I);

	PPa = transform("shader",P) * veining;
	PPa = PPa/2;      /* frequency adjustment (S-shaped curve) */
	pixelsizea = sqrt(area(PPa)); twicea = 2 * pixelsizea;

	/* compute turbulence */
	turbulencea = 0;
	for (scalea = 1; scalea > twicea; scalea /= 2) {
		/**** This function is different - abs() and -0.5 ****/
		turbulencea += scalea * abs(noise(PPa/scalea)-0.5);
	}
	/* gradual fade out of highest freq component near visibility limit */
	if (scalea > pixelsizea) {
		weighta = (scalea / pixelsizea) - 1;
		weighta = clamp(weighta, 0, 1);
		/**** This function is different - abs() and -0.5 ****/
		turbulencea += weighta * scalea * abs(noise(PPa/scalea)-0.5);
	}
	/* Find out average changes to make to marblecolor 
		through veincolor, the ending color 
	*/
	mc = marblecolor; vc = veincolor;
	red=comp(mc,0); green=comp(mc,1); blue=comp(mc,2);
	vred=comp(vc,0); vgreen=comp(vc,1); vblue=comp(vc,2); 
	 cd=vblue-blue-.05;
	 cc=clamp(((vgreen-green)+(cd-.1))/2,-1,1);
	 cb=clamp(((cc-.1)+(cd-.2))/2,-1,1);
	 ca=clamp(((cd-.5)+(cc-.4)+(cb-.3))/3,-1,1);
	 c =clamp(((cd-.6)+(cc-.5)+(cb-.4)+(ca-.1))/4,-1,1);
	 
	/*
	 * turbulence now has a range of 0:2, but its values actually
	 * tend strongly to lie around 0.75 to 1.
	 */

	cmia = turbulencea;
	cmia = clamp(cmia, 0, 1);
	diffusecolora =
		color spline(cmia,
		color (red, green, blue),
		color (red, green, blue),
		color (red, clamp(green + (cc/2),0,1), clamp(blue + (cb/2),0,1)),
		color (clamp(red - ca,0,1), clamp(green + c,0,1), clamp(blue + cb,0,1)),
		color (clamp(red - cb,0,1), clamp(green + c,0,1), clamp(blue + (cd/2),0,1)),
		color (clamp(red - (cd + 0.05),0,1), clamp(green - (cd + 0.05),0,1), clamp(blue + (c/2),0,1)),
		color (red, clamp(green + (cc - 0.01),0,1), clamp(blue + (cd + 0.02),0,1)),
		color (red, clamp(green + cc,0,1), clamp(blue + (cd + 0.05),0,1))
		);
	
	/* use color spline to compute basic color */
	
	Oi = Os;
	Ci = diffusecolora * (Ka*ambient() + Kd*diffuse(Nfa));
	
	/* add in specular component */
	
	Ci = Os * (Ci + specularcolor * Ks * specular(Nfa,Va,roughness));
}
