/*
 * crystalglow(): a shader that provides a increasing intensity glow 
                    towards the edge via (1-falloff) 
 */
surface
crystalglow( float    Ka= 1,
                Kd= 0.1,
                Ks= 1,
                roughness = 0.025,
                maxglow = 3, /* remember ambience increases W.R.T the power 
                                curve made by attenuation to a maximum of 
                                maxglow */
                attenuation = 0.5; /* this is the power to which the glow
                                        is mapped to:
                                        attenuation  clear/color
                                             0       ***********
                                             .5      ********|||
                                             1       ******|||||
                                             2       ***||||||||
                                       Infinity(=20) |||||||||||    
                                    */
       color    specularcolor = 1
     ) 
{
        point NF = faceforward( normalize(N), I);
        point V  = normalize(-I); 
        point NN = normalize(N);
        point II = normalize(I);
        float falloff = II.NN;  /* Direct incidence has cosine closer to 1.*/
        float glow;
        float falloffa = II.NN;
        
        if (falloff < 0) {              /* Back of sphere only */
                /* Normalize falloff by lengths of I and N */
                falloff = falloff * falloff / (II.II * NN.NN) ;
                falloffa = pow(falloff, attenuation);
                glow = (1- falloffa);
                glow = clamp(maxglow - glow, 0, maxglow);
                Ci = ((Ka*ambient()*glow) + (Kd*diffuse(NF))) * Cs *( 1-falloffa);
                Ci+= specularcolor*Ks*specular(NF,V,roughness);
                Oi = 1 - falloffa;
        } else
                Oi = 0;
}
  
