/* marblefloor(): surface shader for applying a marble checkerboard pattern.
                    it has a plethera of params, but can give an AWESOME 
                    effect, like gold and black with white accents and 
                    high specularity and low roughness!
*/

surface 
marblefloor(
        string rfltex       = "";
        string dirttex      = "";
        float   Ka          = 0.3,          /* Previously = 0.1 */
                Kd          = 0.5, 
                Kr          = 0.85,
                Ks          = 0.9,
                Kdirtiness  = 0.4,
                roughness   = 0.0125,
                frequency   = 4,
                veining     = 1.0,
                veinmix     = .25;
        string  coordflip   = "";
        color   specularcolor = 1,
                marblecolor   = 1,
                veincolor     = 1,
                slatecolor    = 1,
                sveincolor    = 1
           )
{
        point PP;            /* scaled point in shader space */
        point PNDC;
        float csp;           /* color spline parameter */
        point Nf;            /* forward-facing normal */
        point V;             /* for specular() */
        float pixelsize, twice, scale, weight, turbulence;
        color anyc, bcolor ;
        float pixelsizea, twicea, scalea, weighta, turbulencea;
        float   smod = mod(s*frequency,1),
                tmod = mod(t*frequency,1);
    varying point PPa;
    float cmia;
    varying point Va, Nfa ;
    color diffusecolora ;
    float c,ca,cb,cc,cd;
    float red,green,blue,vred,vgreen,vblue;
    color mc,vc;
    point Rray;
    color reflection, dirticolor;
        
/* The_marble shader is the main checker patch */        

        float Vm,Vma,Vmb;
        color normalcolor, mediumcolor, mediumdcolor, darkcolor; 
        color ultracolor;

    /* Obtain a forward-facing normalcolor for lighting calculations. */
    Nf = faceforward( normalize(N), I);
    V = normalize(-I);

    PP = transform("shader", P) * veining;
    pixelsize = sqrt(area(PP));
    twice = 2 * pixelsize;
    turbulence = 0;
    for (scale = 1; scale > twice; scale /= 2) 
        turbulence += scale * noise(PP/scale);

    /* Gradual fade out of highest-frequency component near limit */
    if (scale > pixelsize) {
        weight = (scale / pixelsize) - 1;
        weight = clamp(weight, 0, 1);
        turbulence += weight * scale * noise(PP/scale);
    }

    /* mix veincolor in with marblecolor in the same fashion as blue_marble */

    Vm=veinmix; Vma=clamp((Vm*2),0,1); Vmb=clamp((Vm*3),0,1);
    mc=marblecolor;
    vc=veincolor;
    normalcolor= mc;
    mediumcolor= mix(mc,vc,Vm);
    mediumdcolor= mix(mc,vc,Vma);
    darkcolor= mix(mc,vc,Vmb);


    /*
     * Magnify the upper part of the turbulence range 0.75:1
     * to fill the range 0:1 and use it as the parameter of
     * a color spline through various shades of blue.
     */
    csp = clamp(4 * turbulence - 3, 0, 1);
    bcolor = color spline(csp, normalcolor, normalcolor, 
                    mediumcolor, mediumcolor, mediumcolor, normalcolor,
                    normalcolor, mediumdcolor, mediumdcolor, 
                    darkcolor, darkcolor, normalcolor, normalcolor
                    );
/* Anymarble.sl */

    Nfa = faceforward( normalize(N), I);
    Va = -normalize(I);

    PPa = transform("shader",P) * veining;
    PPa = PPa/2;      /* frequency adjustment (S-shaped curve) */
    pixelsizea = sqrt(area(PPa)); twicea = 2 * pixelsizea;

    /* compute turbulence */
    turbulencea = 0;
    for (scalea = 1; scalea > twicea; scalea /= 2) {
        /**** This function is different - abs() and -0.5 ****/
        turbulencea += scalea * abs(noise(PPa/scalea)-0.5);
    }
    /* gradual fade out of highest freq component near visibility limit */
    if (scalea > pixelsizea) {
        weighta = (scalea / pixelsizea) - 1;
        weighta = clamp(weighta, 0, 1);
        /**** This function is different - abs() and -0.5 ****/
        turbulencea += weighta * scalea * abs(noise(PPa/scalea)-0.5);
    }
    /* Find out average changes to make to marblecolor through veincolor, the ending    
       color 
    */
    mc = slatecolor; vc = sveincolor;
    red=comp(mc,0); green=comp(mc,1); blue=comp(mc,2);
    vred=comp(vc,0); vgreen=comp(vc,1); vblue=comp(vc,2); 
     cd=vblue-blue-.05;
     cc=clamp(((vgreen-green)+(cd-.1))/2,-1,1);
     cb=clamp(((cc-.1)+(cd-.2))/2,-1,1);
     ca=clamp(((cd-.5)+(cc-.4)+(cb-.3))/3,-1,1);
     c =clamp(((cd-.6)+(cc-.5)+(cb-.4)+(ca-.1))/4,-1,1);
     
    /*
     * turbulence now has a range of 0:2, but its values actually
     * tend strongly to lie around 0.75 to 1.
     */

    /**** This is different - no multiply by 4 and subtract 3 ****/
    cmia = turbulencea;
    cmia = clamp(cmia, 0, 1);
    diffusecolora =
        color spline(cmia,
        color (red, green, blue),
        color (red, green, blue),
        color (red, clamp(green + (cc/2),0,1), clamp(blue + (cb/2),0,1)),
        color (clamp(red - ca,0,1), clamp(green + c,0,1), clamp(blue + cb,0,1)),
        color (clamp(red - cb,0,1), clamp(green + c,0,1), clamp(blue + (cd/2),0,1)),
        color (clamp(red - (cd + 0.05),0,1), clamp(green - (cd + 0.05),0,1), clamp(blue + (c/2),0,1)),
        color (red, clamp(green + (cc - 0.01),0,1), clamp(blue + (cd + 0.02),0,1)),
        color (red, clamp(green + cc,0,1), clamp(blue + (cd + 0.05),0,1))
        );
    /* use color spline to compute basic color */

    anyc= diffusecolora;
    

/* Reflection texture map read */
    
    if (rfltex !=""){
        PNDC = transform("NDC", P);
        if (coordflip == ""){    
                reflection = color texture(rfltex, (xcomp(PNDC)), (ycomp(PNDC)));
        }else
        if (coordflip == "x"){
                reflection = color texture(rfltex, (1.0-xcomp(PNDC)), (ycomp(PNDC)));
        }else
        if (coordflip == "y"){
                reflection = color texture(rfltex, (xcomp(PNDC)), (1.0-ycomp(PNDC)));
        }else
        if (coordflip == "xy"){
                reflection = color texture(rfltex, (1.0-xcomp(PNDC)), (1.0-ycomp(PNDC)));
        }
            reflection = Kr * reflection;
    }else{
        reflection = 1;
    }
/* Dirtiness texture map read */
    
    if (dirttex !=""){
        dirticolor = Kdirtiness * color texture(dirttex, s, t);
    }else{
        dirticolor = 1;
    }
    
    if(smod < 0.5) {
            if(tmod < 0.5) 
                Ci = anyc;
            else
                Ci = bcolor;
        } else {
            if(tmod < 0.5) 
                Ci = bcolor;
            else
                Ci = anyc;
        }

    Oi = Os;
    Ci = Oi*Ci*reflection*dirticolor*(Ka*ambient() + Kd*diffuse(Nf));
    Ci += (specularcolor*Ks*specular(Nf,Va,roughness));
}
