/* the_marble(): a marble that is completely controllable
 */

surface
the_marble(
           float   Ka    = 0.1,
                   Kd    = 0.6, 
                   Ks    = 0.4, 
                   roughness = 0.1,
                   txtscale  = 1.0,
                   veinmix   = 0.33;
           color   specularcolor = 1,
                   marblecolor =1,
                   veincolor =1
           )
{
    point PP;            /* scaled point in shader space */
    float csp;           /* color spline parameter */
    point Nf;            /* forward-facing normal */
    point V;             /* for specular() */
    float pixelsize, twice, scale, weight, turbulence;
    float Vm,Vma,Vmb;
    color mc,vc;
    color normalcolor, mediumcolor, mediumdcolor, darkcolor; 
    color ultracolor;

    /* Obtain a forward-facing normalcolor for lighting calculations. */
    Nf = faceforward( normalize(N), I);
    V = normalize(-I);

    PP = transform("shader", P) * txtscale;
    pixelsize = sqrt(area(PP));
    twice = 2 * pixelsize;
    turbulence = 0;
    for (scale = 1; scale > twice; scale /= 2) 
        turbulence += scale * noise(PP/scale);

    /* Gradual fade out of highest-frequency component near limit */
    if (scale > pixelsize) {
        weight = (scale / pixelsize) - 1;
        weight = clamp(weight, 0, 1);
        turbulence += weight * scale * noise(PP/scale);
    }

    /* mix veincolor in with marblecolor in the same fashion as blue_marble */

    Vm=veinmix; Vma=clamp((Vm*2),0,1); Vmb=clamp((Vm*3),0,1);
    mc=marblecolor;
    vc=veincolor;
    normalcolor= mc;
    mediumcolor= mix(mc,vc,Vm);
    mediumdcolor= mix(mc,vc,Vma);
    darkcolor= mix(mc,vc,Vmb);


    /*
     * Magnify the upper part of the turbulence range 0.75:1
     * to fill the range 0:1 and use it as the parameter of
     * a color spline through various shades of blue.
     */
    csp = clamp(4 * turbulence - 3, 0, 1);
    Ci = color spline(csp, normalcolor, normalcolor, 
                    mediumcolor, mediumcolor, mediumcolor, normalcolor,
                    normalcolor, mediumdcolor, mediumdcolor, 
                    darkcolor, darkcolor, normalcolor, normalcolor
                    );

    /* Multiply this color by the diffusely reflected light. */
    Ci *= Ka*ambient() + Kd*diffuse(Nf);

    /* Adjust for opacity. */
    Oi = Os;
    Ci = Ci * Oi;

    /* Add in specular highlights. */
    Ci += specularcolor * Ks * specular(Nf,V,roughness);
}
